/*
 main.cpp - Device Firmware Update Utility

 Copyright (c) 2021 Sasapea's Lab. All right reserved.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General
 Public License along with this library; if not, write to the
 Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 Boston, MA  02111-1307  USA
 */
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <errno.h>

#include "firmware.h"
#include "i16hex.h"

void
usage(void)
{
  printf("\n");
  printf("Device Firmeware Update Utility, Version 1.0\n");
  printf("Copyright(c) 2021 Sasapea's Lab. All right reserved.\n");
  printf("\n");
  printf("Usage: firmware port-name [options] intelhex-file\n");
  printf("\n");
  printf("  options: -reset ... reset bootloader\n");
  printf("           -erase ... erase program\n");
  printf("\n");
  exit(EXIT_FAILURE);
}

int
main(int argc, char **argv)
{
  bool success = false;

  if (argc >= 3)
  {
    I16Hex   ih;
    Firmware fw;
    const char *dev = argv[1];
    const char *src = argv[2];
    if (fw.open(dev) < 0)
      fw.error("Firmware.open(\"%s\"): %s\n", dev, strerror(errno));
    else
    {
      if (*src == '-')
      {
        if (strcmp(src, "-reset") == 0)
          success = fw.reset();
        else if (strcmp(src, "-erase") == 0)
          success = fw.erase();
      }
      else
      {
        if (!ih.load(src))
          fw.error("I16Hex.load(\"%s\"): %s\n", src, strerror(errno));
        else
          success = fw.upload(ih.bytes(), ih.start(), ih.end());
      }
      fw.close();
    }
  }

  if (!success)
    usage();

  return EXIT_SUCCESS;
}
